@@ -1,16 +1,21 @@
+from __future__ import unicode_literals
+
 import logging
 import tempfile
 import os
+import sys
 
-from zest.releaser.utils import system
+from zest.releaser.utils import fs_to_text
+from zest.releaser.utils import execute_command
 from zest.releaser.vcs import BaseVersionControl
 
-logger = logging.getLogger('mercurial')
+logger = logging.getLogger(__name__)
 
 
 class Hg(BaseVersionControl):
     """Command proxy for Mercurial"""
     internal_filename = '.hg'
+    setuptools_helper_package = 'setuptools_hg'
 
     @property
     def name(self):
@@ -20,14 +25,15 @@ class Hg(BaseVersionControl):
         # No setup.py? With hg we can probably only fall back to the directory
         # name as there's no svn-url with a usable name in it.
         dir_name = os.path.basename(os.getcwd())
+        dir_name = fs_to_text(dir_name)
         return dir_name
 
     def available_tags(self):
-        tag_info = system('hg tags')
+        tag_info = execute_command(['hg', 'tags'])
         tags = [line[:line.find(' ')] for line in tag_info.split('\n')]
         tags = [tag for tag in tags if tag]
         tags.remove('tip')  # Not functional for us
-        logger.debug("Available tags: %r", tags)
+        logger.debug("Available tags: '%s'", ', '.join(tags))
         return tags
 
     def prepare_checkout_dir(self, prefix):
@@ -40,33 +46,38 @@ class Hg(BaseVersionControl):
         return version
 
     def cmd_diff(self):
-        return 'hg diff'
+        return ['hg', 'diff']
 
     def cmd_commit(self, message):
-        return 'hg commit -v -m "%s"' % message
+        return ['hg', 'commit', '-v', '-m', message]
 
     def cmd_diff_last_commit_against_tag(self, version):
-        current_revision = system('hg identify')
+        current_revision = execute_command(['hg', 'identify'])
         current_revision = current_revision.split(' ')[0]
         # + at the end of the revision denotes uncommitted changes
         current_revision = current_revision.rstrip('+')
-        return "hg diff -r %s -r %s" % (version, current_revision)
+        return ['hg', 'diff', '-r', version, '-r', current_revision]
 
     def cmd_log_since_tag(self, version):
-        current_revision = system('hg identify')
+        current_revision = execute_command(['hg', 'identify'])
         current_revision = current_revision.split(' ')[0]
         # + at the end of the revision denotes uncommitted changes
         current_revision = current_revision.rstrip('+')
-        return "hg log -r %s -r %s" % (version, current_revision)
+        return ['hg', 'log', '-r', version, '-r', current_revision]
 
-    def cmd_create_tag(self, version):
+    def cmd_create_tag(self, version, message, sign=False):
+        if sign:
+            logger.error(
+                "Signing tags with mercurial is not yet implemented, sorry. "
+                "Please check your configuration in 'setup.cfg'.")
+            sys.exit(21)
         # Note: place the '-m' before the argument for hg 1.1 support.
-        return 'hg tag -m "Tagging %s" %s' % (version, version)
+        return ['hg', 'tag', '-m', message, version]
 
     def cmd_checkout_from_tag(self, version, checkout_dir):
-        source = self.workingdir
+        source = self.reporoot
         target = checkout_dir
-        return 'hg clone -r %s %s %s' % (version, source, target)
+        return ['hg', 'clone', '-r', version, source, target]
 
     def checkout_from_tag(self, version):
         package = self.name
@@ -75,7 +86,7 @@ class Hg(BaseVersionControl):
         # just in the current directory.
         tagdir = tempfile.mktemp(prefix=prefix)
         cmd = self.cmd_checkout_from_tag(version, tagdir)
-        print system(cmd)
+        print(execute_command(cmd))
         os.chdir(tagdir)
 
     def is_clean_checkout(self):
@@ -83,11 +94,15 @@ class Hg(BaseVersionControl):
         """
         # The --quiet option ignores untracked (unknown and ignored)
         # files, which seems reasonable.
-        if system('hg status --quiet'):
+        if execute_command(['hg', 'status', '--quiet']):
             # Local changes.
             return False
         return True
 
     def push_commands(self):
         """Return commands to push changes to the server."""
-        return ['hg push']
+        return [['hg', 'push']]
+
+    def list_files(self):
+        """List files in version control."""
+        return execute_command(['hg', 'locate', '--fullpath']).splitlines()
